#define White 0xffffff

#define color_bank(value) out_AttrController(0x14,value);

#define set_color(value) { out_GfxController(0,value);\
			   out_GfxController(1,0xf); }

#define mask_int(master,slave)  asm {  \
				  mov al,master; \
				  out 21h,al;  \
				  mov al,slave; \
				  out 0a1h,al }

#define set_plane(value) out_Sequencer(2,value)

struct VESAInfoStruct
{
char signature[4];
WORD VerNum;
char far *OEMPointer;
DWORD Capabilities;
WORD far *VModeList;
WORD VideoMemin64kb;
BYTE Reserved[237];
} VESAInfo;

struct VESAModeStruct
{
WORD ModeAttrib;	//Merkmale des Videomodus
BYTE Win1Attrib;	//Merkmale 1.Speicherfensters
BYTE Win2Attrib;	//Merkmale 2.Speicherfensters
WORD WinGranularity;	//Auflsung der Fenster in 4KB
WORD WinSize;		//Gre der Fenster in 1 KB
WORD Win1StartSeg;	//Startsegment vom 1.Fenster
WORD Win2StartSeg;	//Startsegment vom 1.Fenster
void far *WinRoutine;	//Einsprung zur Fensterroutine
WORD BytesPerLine;	//Bytes pro Bildzeile
//VESA Modi knnen, OEM Modi mssen die folgenden
//Variablen belegen
WORD Width;		//Breite des Videomodus
WORD Height;		//Hhe des Videomodus
BYTE WOfCBox;		//Breite der Zeichenbox
BYTE HOfCBox;		//Hhe der Zeichenbox
BYTE NumMemPlanes;	//Anzahl der Bitfelder
BYTE BitsPerPixel;	//Bits pro Pixel
BYTE Banks;		//Anzahl der Bnke
BYTE MemModelType;	//Speicheradressierungstyp
BYTE BankSize;		//Gre der Bank
BYTE ImagePages;	//zustzliche Bildschirmseiten
BYTE Reserved1;		//reserviert
//im VESA 1.2 Standard definierte Variablen
BYTE RedMaskSize;	//Gre der roten Maske
BYTE RedMaskPos;	//Position der roten Maske
BYTE GreenMaskSize;	//Gre der grnen Maske
BYTE GreenMaskPos;	//Position der grnen Maske
BYTE BlueMaskSize;	//Gre der blauen Maske
BYTE BlueMaskPos;	//Position der blauen Maske
BYTE ResMaskSize;	//Gre der reservierten Maske
BYTE ResMaskPos;	//Position der reservierten Maske
BYTE DCModeInfo;	//zustzliche Merkmale
BYTE Reserverd2[216];
} ModeInfo;

struct LogScanLine
{
WORD BPScanLine;
WORD NOPixPScanLine;
WORD MaxNumOScanLine;
} LScanLine;

struct DisplayStart
{
WORD LDispPixel;
WORD FScanLine;
} DispStart;

WORD VideoModeInfoNum;
WORD far *ModeList;
WORD Window1;
BYTE far *TextScreen = (BYTE far *) MK_FP(0xb800,0);

struct REGPACK myregs;

BYTE GetVESAInfo(void)
{
myregs.r_ax=0x4f00;
myregs.r_di=FP_OFF(&VESAInfo);
myregs.r_es=FP_SEG(&VESAInfo);
intr(0x10,&myregs);
return(myregs.r_ax>>8);
}


BYTE GetVideoModeInfo(WORD mode)
{
myregs.r_ax=0x4f01;
myregs.r_cx=mode;
myregs.r_di=FP_OFF(&ModeInfo);
myregs.r_es=FP_SEG(&ModeInfo);
intr(0x10,&myregs);
if (myregs.r_ax==0x4f) VideoModeInfoNum=mode;
return(myregs.r_ax>>8);
}

BYTE SetVideoMode(WORD mode,BYTE ForgetIt)
{
  if ((GetVideoModeInfo(mode)==0)||ForgetIt)
  {
    myregs.r_ax=0x4f02;
    myregs.r_bx=mode;
    myregs.r_cx=0;
    myregs.r_dx=0;
    intr(0x10,&myregs);
    Window1=0xffff;
    return(myregs.r_ax>>8);
  }
  else return(1);
}

BYTE SetLogScanLineLen(WORD ScanLineLen)
{
myregs.r_ax=0x4f06;
myregs.r_bx=0;
myregs.r_cx=ScanLineLen;
intr(0x10,&myregs);
ModeInfo.Width=myregs.r_cx;
LScanLine.BPScanLine=myregs.r_bx;
LScanLine.NOPixPScanLine=myregs.r_cx;
LScanLine.MaxNumOScanLine=myregs.r_dx;
return(myregs.r_ax>>8);
}

void init_palette(BYTE red, BYTE green, BYTE blue)
{
  int i,j;
  BYTE pal[384];
  BYTE pos[8]={1,3,2,6,4,12,8,9},
       fac[8]={1,255,1,1,1,1,1,1};

  //Paletten- und Farbregister harmonisieren
  for(i=0;i<16;i++) out_AttrController(i,i);
  for(i=0;i<128;i++)
  {
    pal[3*i]=0;		   //alles schwarz
    pal[3*i+1]=0;		   //alles schwarz
    pal[3*i+2]=0;		   //alles schwarz
    if ((i & pos[i/16]) != 0) {    pal[3*i]=red/fac[i/16];
				 pal[3*i+1]=green/fac[i/16];
				 pal[3*i+2]=blue/fac[i/16];  }
  }
  Set_Palette(0,127,pal); 	   //Palette setzen
  in_AttrController(0x10,i);	   //MODE-Reg lesen
  i|=0x80;			   //16x16 Farbbnke
  out_AttrController(0x10,i);	   //MODE-Reg setzen
  out_AttrController(0x14,0x0);    //Farbbank 0 benutzen
}

void PutString(int x,int y,char *String, char Color)
{
  set_color(Color);
  while (*String!='\0') //solange nicht das Ende des Strings erreicht ist
  {
    put_char(x,y,*String);
    x+=8; //eine Spalte weiter
    String++; //ein Zeichen weiter
  }
}


void line(int x1,int y1,int x2,int y2)
/****************************************************************************
*
* Function:     line
* Parameters:   x1,y1       - First endpoint of line
*               x2,y2       - Second endpoint of line
*
* Description:  Scan convert a line segment using the MidPoint Digital
*               Differential Analyser algorithm.
*
****************************************************************************/
{
    int     d;                      /* Decision variable                */
    int     dx,dy;                  /* Dx and Dy values for the line    */
    int     Eincr,NEincr;           /* Decision variable increments     */
    int     yincr;                  /* Increment for y values           */
    int     t;                      /* Counters etc.                    */

    dx = abs(x2 - x1);
    dy = abs(y2 - y1);

    if (dy <= dx) {

	/* We have a line with a slope between -1 and 1
	 *
	 * Ensure that we are always scan converting the line from left to
	 * right to ensure that we produce the same line from P1 to P0 as the
	 * line from P0 to P1.
	 */

	if (x2 < x1) {
            t = x2; x2 = x1; x1 = t;    /* Swap X coordinates           */
            t = y2; y2 = y1; y1 = t;    /* Swap Y coordinates           */
            }

        if (y2 > y1)
            yincr = 1;
        else
            yincr = -1;

        d = 2*dy - dx;              /* Initial decision variable value  */
        Eincr = 2*dy;               /* Increment to move to E pixel     */
        NEincr = 2*(dy - dx);       /* Increment to move to NE pixel    */

	putpx16_0(x1,y1);      /* Draw the first point at (x1,y1)  */

        /* Incrementally determine the positions of the remaining pixels
         */

        for (x1++; x1 <= x2; x1++) {
            if (d < 0) {
                d += Eincr;         /* Choose the Eastern Pixel         */
		}
	    else {
		d += NEincr;        /* Choose the North Eastern Pixel   */
		y1 += yincr;        /* (or SE pixel for dx/dy < 0!)     */
		}
	    putpx16_0(x1,y1);  /* Draw the point			 */
	    }
	}
    else {

	/* We have a line with a slope between -1 and 1 (ie: includes
	 * vertical lines). We must swap our x and y coordinates for this.
	 *
	 * Ensure that we are always scan converting the line from left to
	 * right to ensure that we produce the same line from P1 to P0 as the
	 * line from P0 to P1.
	 */

	if (y2 < y1) {
	    t = x2; x2 = x1; x1 = t;    /* Swap X coordinates           */
	    t = y2; y2 = y1; y1 = t;    /* Swap Y coordinates           */
	    }

	if (x2 > x1)
	    yincr = 1;
	else
	    yincr = -1;

	d = 2*dx - dy;              /* Initial decision variable value  */
	Eincr = 2*dx;               /* Increment to move to E pixel     */
	NEincr = 2*(dx - dy);       /* Increment to move to NE pixel    */

	putpx16_0(x1,y1);      /* Draw the first point at (x1,y1)  */

	/* Incrementally determine the positions of the remaining pixels
	 */

	for (y1++; y1 <= y2; y1++) {
	    if (d < 0) {
		d += Eincr;         /* Choose the Eastern Pixel         */
		}
	    else {
		d += NEincr;        /* Choose the North Eastern Pixel   */
		x1 += yincr;        /* (or SE pixel for dx/dy < 0!)     */
		}
	    putpx16_0(x1,y1);  /* Draw the point			 */
	    }
	}
}

WORD GetCPUMemControl(BYTE Window)
{
myregs.r_ax=0x4f05;
myregs.r_bx=0x100+Window;
intr(0x10,&myregs);
return(myregs.r_dx);
}

BYTE SetCPUMemControl(BYTE Window,WORD WinAddress)
{
myregs.r_ax=0x4f05;
myregs.r_bx=Window;
myregs.r_dx=WinAddress;
intr(0x10,&myregs);
return(myregs.r_ax>>8);
}

void InitMem(void)
{
  WORD Mem,ActMem=0,i;

  Mem=VESAInfo.VideoMemin64kb*64; //Speichergre
  for(i=0;ActMem<Mem;i++,ActMem+=ModeInfo.WinSize)
  { //Fenster fr Fenster initialisieren
    SetCPUMemControl(0,i*ModeInfo.WinSize/
		     ModeInfo.WinGranularity);
    _fmemset(GfxScreen,0,ModeInfo.WinSize*1024-1);
  }
}

DWORD TestOffs(DWORD Offs)
{
  DWORD window=0, WSize;
  WSize=(DWORD) ModeInfo.WinSize*1024;
  window=Offs/WSize;
  if (Window1!=window)
  {
    SetCPUMemControl(0, window*ModeInfo.WinSize/ModeInfo.WinGranularity);
    Window1=window;
  }
  return(Offs);
}
